/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  This file is part of the ZipCracker Program.

    Copyright (C) 1998-2000 Jonas Borgstrm <jonas_b@bitsmart.com>.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
#include <gnome.h>
#include <glib.h>
#include <gnome-xml/tree.h>
#include <gnome-xml/parser.h>

#include "zipcracker.h"
#include "mainwin.h"
#include "zip.h"
#include "crack_server.h"
#include "targetfile.h"
/****************************************************************************/
#include "logo.xpm"
/****************************************************************************/
#define ZC_BUTTON_PREV 0
#define ZC_BUTTON_QUIT 1
#define ZC_BUTTON_HELP 2
#define ZC_BUTTON_NEXT 3

enum PageId {
	WelcomePage = 0,
	PasswordPage,
	CrackPage
};
/****************************************************************************/
static GtkWidget *create_zipfile_page ();
static GtkWidget *create_password_page ();
static GtkWidget *create_crack_page ();
void zipcracker_init_window ();
static gboolean next_cb (GtkWidget * widget, gpointer data);
static gboolean prev_cb (GtkWidget * widget, gpointer data);
static void save_to_xml ();
static gboolean load_from_xml ();
static void select_zipfile_cb (GtkWidget * widget, gpointer data);
static void select_zipfile_destroy_cb (GtkWidget * widget, GtkWidget **data);
static void select_zipfile_ok_cb (GtkWidget * widget, GtkWidget *fs);
static void select_zipfile_cancel_cb (GtkWidget * widget, GtkWidget *fs);
static void checkbox_custom_toggled_cb (GtkWidget * widget, gpointer data);
static void checkbox_startpw_toggled_cb (GtkWidget * widget, gpointer data);
static void checkbox_format_toggled_cb (GtkWidget * widget, gpointer data);
static gboolean check_zipfile ();
static int update_crack_page ();
static void about (GtkWidget *widget, gpointer data);
static void zipcracker_continue (GtkWidget *widget, gpointer);
static gint crackpage_timeout (gpointer data);
static void free_entry (gpointer data, gpointer userdata);
/****************************************************************************/
Properties *prop = NULL;
MainWindow *mw = NULL;
/****************************************************************************/
extern GList *zc_todo;
/****************************************************************************/
/* Saves the current state to a xml file */
/****************************************************************************/
static void
save_to_xml ()
{
	xmlDocPtr doc;
	xmlNodePtr tree, subtree;
	xmlNsPtr zipcracker_ns;
	GString *filename;
	GList *list;
	ZCTodo *t;

        g_assert (prop != NULL);
	
	doc = xmlNewDoc("1.0");
	doc->root = xmlNewDocNode(doc, NULL, "Savedata", NULL);
	zipcracker_ns = xmlNewNs (doc->root, 
				  "http://zipcracker.sourceforge.net/", "zipcracker");
	xmlSetNs (doc->root, zipcracker_ns);

	subtree = xmlNewChild(doc->root, NULL, "Filename", prop->zipfilename->str);
	tree = xmlNewChild(doc->root, NULL, "PasswordString", NULL);

	xmlSetProp(tree, "az", 
		   gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(mw->checkbox_az)) ? "1" : "0");
	xmlSetProp(tree, "AZ", 
		   gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(mw->checkbox_AZ)) ? "1" : "0");
	xmlSetProp(tree, "ZeroToNine", 
		   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mw->checkbox_09)) ? "1" : "0");

	xmlSetProp(tree, "custom", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mw->checkbox_custom)) ? 
		   gtk_entry_get_text(GTK_ENTRY(mw->entry_custom)) : "");
	xmlSetProp(tree, "format", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mw->checkbox_format)) ? 
		   gtk_entry_get_text(GTK_ENTRY(mw->entry_format)) : "");

	subtree = xmlNewChild(doc->root, NULL, "CurrentPassword", zc_crack_get_current_password ());

	/* Save the "todo" blocks */
	list = zc_todo;
	while (list) {

		t = (ZCTodo *)list->data;

		subtree = xmlNewChild(doc->root, NULL, "TodoBlock", NULL);
		xmlSetProp(subtree, "start", t->start);
		xmlSetProp(subtree, "end", t->end);

		list = g_list_next (list);
	}

	/* Create the filename */
	filename = g_string_new ("");
	g_string_sprintf (filename, "%s%c.zipcracker_save.xml", g_get_home_dir (), G_DIR_SEPARATOR);

	if (xmlSaveFile (filename->str, doc) == -1)
		gtk_widget_show (gnome_message_box_new ("Failed to save ~/.zipcracker_save.xml", 
							GNOME_MESSAGE_BOX_ERROR, GNOME_STOCK_BUTTON_OK, 
							NULL));

	g_string_free (filename, TRUE);
	xmlFreeDoc (doc);
}
/****************************************************************************/
/* frees the data variable, used with g_list_foreach */
/****************************************************************************/
static void
free_data (gpointer data, gpointer userdata)
{
	g_free (data);
}
/****************************************************************************/
/* Loads the state from a xml file */
/****************************************************************************/
static gboolean
load_from_xml ()
{
	xmlDocPtr doc;
	xmlNodePtr cur;
	xmlNsPtr ns;
	xmlChar *ptr;
	GString *filename = g_string_new ("");

	/* Create the filename */
	g_string_sprintf (filename, "%s%c.zipcracker_save.xml", 
			  g_get_home_dir (), G_DIR_SEPARATOR);

	doc = xmlParseFile(filename->str);

	/* Free unused memory */
	g_string_free (filename, TRUE);

	if (doc == NULL) {
		
		return FALSE;
	}
	/*
	 * Check the document is of the right kind
	 */
	cur = doc->root;
	if (cur == NULL) {

		xmlFreeDoc(doc);
		return FALSE;
	}
	ns = xmlSearchNsByHref(doc, cur, "http://zipcracker.sourceforge.net/");
	if (ns == NULL) {

		xmlFreeDoc(doc);
		return FALSE;
	}
	if (strcmp(cur->name, "Savedata")) {

		xmlFreeDoc(doc);
		return FALSE;
	}
	/* Free old "todo" list */
	g_list_foreach (zc_todo, free_data, NULL);
	g_list_free (zc_todo);
	zc_todo = NULL;

	cur = cur->childs;
	while (cur != NULL) {
		if ((!strcmp(cur->name, "Filename")) && (cur->ns == ns)) {
			ptr = xmlNodeListGetString (doc, cur->childs, 1);
			if (ptr) {

				g_string_assign (prop->zipfilename, ptr);
				g_free (ptr);
			}
		}
		else if ((!strcmp(cur->name, "TodoBlock")) && (cur->ns == ns)) {
			gchar *start, *end;
			start = xmlGetProp (cur, "start");
			end = xmlGetProp (cur, "end");

			if (start && end) {
				ZCTodo *t = g_new (ZCTodo, 1);
				
				strcpy (t->start, start);
				strcpy (t->end, end);
				zc_todo = g_list_append (zc_todo, t);
			}
		}
		else if ((!strcmp(cur->name, "PasswordString")) && (cur->ns == ns)) {
			ptr = xmlGetProp (cur, "az");
			if (ptr) {

				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mw->checkbox_az), 
							      *ptr == '1' ? TRUE : FALSE);
				g_free (ptr);
			} else
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mw->checkbox_az), 
							      FALSE);

			ptr = xmlGetProp (cur, "AZ");
			if (ptr) {

				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mw->checkbox_AZ), 
							      *ptr == '1' ? TRUE : FALSE);
				g_free (ptr);
			} else
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (mw->checkbox_AZ), FALSE);

			ptr = xmlGetProp (cur, "ZeroToNine");
			if (ptr) {

				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mw->checkbox_09), 
							      *ptr == '1' ? TRUE : FALSE);
				g_free (ptr);
			} else
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (mw->checkbox_09), FALSE);

			ptr = xmlGetProp (cur, "custom");
			if (ptr) {
				if (strlen (ptr)) {
					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mw->checkbox_custom), TRUE);
					gtk_entry_set_text (GTK_ENTRY (mw->entry_custom), ptr);
				} else
					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mw->checkbox_custom), FALSE);
				g_free (ptr);
			} else
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mw->checkbox_custom), FALSE);
			ptr = xmlGetProp (cur, "format");
			if (ptr) {

				if (strlen (ptr)) {

					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mw->checkbox_format), TRUE);
					gtk_entry_set_text (GTK_ENTRY (mw->entry_format), ptr);
				} else
					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mw->checkbox_format), FALSE);

				g_free (ptr);
			} else
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (mw->checkbox_format), FALSE);
		}
		else if ((!strcmp(cur->name, "CurrentPassword")) && (cur->ns == ns)) {
			ptr = xmlNodeListGetString (doc, cur->childs, 1);
			if (ptr) {

				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (mw->checkbox_startpw), TRUE);
				gtk_entry_set_text (GTK_ENTRY (mw->entry_startpw), ptr);
				g_free (ptr);
			}
		}
		cur = cur->next;
	}
	xmlFreeDoc(doc);

	return TRUE;
}
/****************************************************************************/
/* Close Application */
/****************************************************************************/
static void
quit_cb (GtkWidget *widget, void *data)
{
	/* Save the session if we are cracking */
	if (gtk_notebook_current_page (GTK_NOTEBOOK (mw->notebook)) == CrackPage) {

		zc_crack_stop ();
		save_to_xml ();
	}

	gtk_main_quit ();

	return;
}
/****************************************************************************/
/* Program entry point */
/****************************************************************************/
int
main (int argc, char *argv[])
{
	gnome_init("zipcracker", VERSION, argc, argv);
	
	/* Initialize localization */
	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
	textdomain (PACKAGE);

	/* Create UI */   
	zipcracker_init_window ();

	/* Use the filename on the command line if specified */
	if(argc == 2) {
		
		g_string_assign (prop->zipfilename, argv[1]);
		next_cb(NULL, NULL);
	}
	
	gtk_main();
	
	return 0;
}
/****************************************************************************/
/* Create Application Window */
/****************************************************************************/
void
zipcracker_init_window ()
{
	GtkWidget *vbox;
	GtkWidget *label;
	
	mw = g_malloc0(sizeof(MainWindow));
	prop = g_malloc0(sizeof(Properties));

	prop->zipfilename = g_string_new("");

	mw->window = gnome_dialog_new(_("ZipCracker"),
				GNOME_STOCK_BUTTON_PREV,
				GNOME_STOCK_BUTTON_CLOSE,
				GNOME_STOCK_BUTTON_HELP,
				GNOME_STOCK_BUTTON_NEXT,
				NULL);
				
	gnome_dialog_set_sensitive(GNOME_DIALOG(mw->window),
		ZC_BUTTON_PREV, FALSE);
		
	gnome_dialog_set_sensitive(GNOME_DIALOG(mw->window),
		ZC_BUTTON_NEXT, FALSE);
				
	/* Connecting buttons */
	gnome_dialog_button_connect(GNOME_DIALOG(mw->window),
		ZC_BUTTON_PREV,
		(GtkSignalFunc) prev_cb,
		NULL);
		
	gnome_dialog_button_connect(GNOME_DIALOG(mw->window),
		ZC_BUTTON_QUIT,
		(GtkSignalFunc) quit_cb,
		NULL);
		
	gnome_dialog_button_connect(GNOME_DIALOG(mw->window),
		ZC_BUTTON_NEXT,
		(GtkSignalFunc) next_cb,
		NULL);
		
	gtk_signal_connect (GTK_OBJECT (mw->window), "delete_event",
			    GTK_SIGNAL_FUNC (quit_cb),
			    NULL);
	
	gtk_widget_realize(mw->window);
	
	vbox = gtk_vbox_new(FALSE, 0);
	
	gtk_box_pack_start( GTK_BOX(GNOME_DIALOG(mw->window)->vbox), vbox,
		TRUE, TRUE, 0);
		
	gtk_widget_show(vbox);
	
	mw->notebook = gtk_notebook_new();

	gtk_container_set_border_width(
		GTK_CONTAINER(mw->notebook), 10);
		
	gtk_box_pack_start(GTK_BOX(vbox), mw->notebook,
		TRUE, TRUE, 0);
	
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(mw->notebook), FALSE);
	gtk_notebook_set_show_border(GTK_NOTEBOOK(mw->notebook), FALSE);
	gtk_widget_show(mw->notebook);
	
	label = gtk_label_new("Zipfile");
	gtk_notebook_append_page(GTK_NOTEBOOK(mw->notebook),
		create_zipfile_page(),
		label);

	gtk_widget_show(label);

	label = gtk_label_new("Passwordtype");
	gtk_notebook_append_page(GTK_NOTEBOOK(mw->notebook),
		create_password_page(),
		label);
		
	gtk_widget_show(label);

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mw->checkbox_az), TRUE);
	label = gtk_label_new("Crack");
	gtk_notebook_append_page(GTK_NOTEBOOK(mw->notebook),
				 create_crack_page (),
				 label);

	gtk_widget_show(label);
	
	gtk_widget_show(mw->window);
}
/****************************************************************************/
/* Allows the user to continue on an aborted job */
/****************************************************************************/
static void zipcracker_continue (GtkWidget *widget, gpointer data)
{
	if (!load_from_xml ()) {
		
		gtk_widget_show (gnome_message_box_new (_("Faild to open ~/.zipcracker_save.xml"), 
							GNOME_MESSAGE_BOX_ERROR, GNOME_STOCK_BUTTON_OK, 
							NULL));
		return;
	}
	/* Switch to the crack page */
	if (next_cb (NULL, NULL))
		next_cb (NULL, NULL);
}
/****************************************************************************/
/* Displays an about box */
/****************************************************************************/
static void about (GtkWidget *widget, gpointer data)
{
	const gchar *authors[] = {"Jonas Borgstrm", NULL};
	
	GtkWidget *about = gnome_about_new("ZipCracker", VERSION,
					   "Copyright (c) 1999-2000 Jonas Borgstrm", 
					   authors, 
					   "http://zipcracker.sourceforge.net/", 
					   NULL);
	
	gtk_widget_show(about);
}
/****************************************************************************/
/* Check to se if it is a valid zipfile */
/****************************************************************************/
static gboolean
check_zipfile ()
{
	GString *message;	

	if(prop->list) {
		/* Delete old list */
		g_list_foreach(prop->list, free_entry, NULL);
		g_list_free(prop->list);
	}
	/* Open zipfile and get filenames */
	prop->list = parse_zipfile(prop->zipfilename->str);
	
	if(prop->list == NULL) {
	
		message = g_string_new("");

		g_string_sprintf(message, _("Can't open\n'%s'.\nmaybe it's not a zip archive."), 
				 prop->zipfilename->str);
		
		gtk_widget_show (gnome_message_box_new(	message->str, 
							GNOME_MESSAGE_BOX_ERROR, 
							GNOME_STOCK_BUTTON_OK, 
							NULL));
		
		g_string_free(message, TRUE);
		
		return 0;
	}
	
	if((prop->targetfile = targetfile_get_best (prop->list)) == NULL) {
		/* Oops no encryped files? */
		
		message = g_string_new("");
		g_string_sprintf(message, 
			_("The file:\n'%s'\ncontains no encryped files."), 
			prop->zipfilename->str);
		
		gtk_widget_show(gnome_message_box_new(
			message->str, 
			GNOME_MESSAGE_BOX_ERROR, 
			GNOME_STOCK_BUTTON_OK, 
			NULL));

		g_string_free(message, TRUE);
		
		return FALSE;
	}

	return TRUE;
}
/****************************************************************************/
/* Returns the password string if any else NULL, call */
/****************************************************************************/
gchar * zc_get_password_string ()
{
	if (prop->password_string == NULL)
		prop->password_string = g_string_new ("");

	g_string_assign (prop->password_string, "");
	
	if(gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (mw->checkbox_az))) {
	
		g_string_append (prop->password_string, 
				 "abcdefghijklmnopqrstuvwxyz");
	}
	if(gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (mw->checkbox_AZ))) {
	
		g_string_append (prop->password_string, 
			"ABCDEFGHIJKLMNOPQRSTUVWXYZ");
	}
	if(gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (mw->checkbox_09))) {
	
		g_string_append (prop->password_string, "0123456789");
	}

	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (mw->checkbox_custom))) {
	
		g_string_append (prop->password_string,
			gtk_entry_get_text (GTK_ENTRY (mw->entry_custom)));
	}
	return prop->password_string->str;
}
/****************************************************************************/
/* Returns the password format */
/****************************************************************************/
gchar * zc_get_password_format ()
{
	if (prop->password_format == NULL)
		prop->password_format = g_string_new ("");
	
	g_string_append (prop->password_format, 
			 gtk_entry_get_text (GTK_ENTRY (mw->entry_format)));
	
	
	if (prop->password_format->len == 0)
		g_string_assign (prop->password_format, "*");

	return prop->password_format->str;
}
/****************************************************************************/
/* Returns the start password */
/****************************************************************************/
gchar * zc_get_start_password ()
{
	if (prop->start_password == NULL)
		prop->start_password = g_string_new ("");
	
	g_string_assign (prop->start_password, "");

	if(gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (mw->checkbox_startpw)))	
		g_string_assign (prop->start_password, 
				 gtk_entry_get_text (GTK_ENTRY (mw->entry_startpw)));

	if (prop->start_password->len == 0)
		g_string_append_c (prop->start_password, *zc_get_password_string ());

	return prop->start_password->str;
}
/****************************************************************************/
/* Initializes the cracking and starts the thread */
/****************************************************************************/
static int
update_crack_page ()
{
	GString *message;
	GString *filename;
	gint i;
	gchar *start_string, *start_passwd;
	
	/* Store the filename without path in "filename" */
	if(strrchr (prop->zipfilename->str, '/')) {
		filename = g_string_new((gchar *)(
			strrchr (prop->zipfilename->str, '/') + 1));
	} else {
		filename = g_string_new(prop->zipfilename->str);
	}
	
	message = g_string_new("");
	
	g_string_sprintf(message, _("Cracking '%s'"), filename->str);
		
	/* Update label */
	gtk_label_set_text (GTK_LABEL(mw->crack_label_filename), 
		message->str);

	if(strlen (zc_get_password_string ()) == 0) {
			gtk_widget_show(gnome_message_box_new(
					_("You must select at least\nOne of the password type checkboxes"), 
					GNOME_MESSAGE_BOX_ERROR, 
					GNOME_STOCK_BUTTON_OK, 
					NULL));
		return 1;
	}
	/* Don't want a start password with invalid characters */
	start_passwd = zc_get_start_password ();
	start_string = zc_get_password_string ();
	i = 0;
	while (start_passwd[i]) {
		if (strchr (start_string, start_passwd[i]) == NULL) {

			gtk_widget_show(gnome_message_box_new(
					_("'Start' contains characters not\nin 'the password characters'"), 
					GNOME_MESSAGE_BOX_ERROR, 
					GNOME_STOCK_BUTTON_OK, 
					NULL));

			return 1;
		}
		i++;
	}

	g_string_free(message, TRUE);
	g_string_free(filename, TRUE);

	zc_crack_start (prop->targetfile);

	return 0;
}
/****************************************************************************/
/* Switch to the next page */
/****************************************************************************/
static gboolean
next_cb (GtkWidget * widget, gpointer data)
{
	switch (gtk_notebook_current_page (GTK_NOTEBOOK (mw->notebook)) + 1)
	{
	case PasswordPage:
		/* Check to see if the file is a encrypted zipfile */
		if (!check_zipfile ())
			return FALSE;

		gnome_dialog_set_sensitive(GNOME_DIALOG(mw->window),
			ZC_BUTTON_NEXT, TRUE);
		break;
	
	case CrackPage:
		if(update_crack_page()) 
			return FALSE;
		gnome_dialog_set_sensitive(GNOME_DIALOG(mw->window),
			ZC_BUTTON_NEXT, FALSE);
		break;
	};
	
	gnome_dialog_set_sensitive(GNOME_DIALOG(mw->window),
		ZC_BUTTON_PREV, TRUE);
		
	gtk_notebook_next_page (GTK_NOTEBOOK (mw->notebook));

	return TRUE;
}
/****************************************************************************/
/* Switch to the previous page */
/****************************************************************************/
static gboolean
prev_cb (GtkWidget * widget, gpointer data)
{
	switch (gtk_notebook_current_page (GTK_NOTEBOOK (mw->notebook)))
	{
	case PasswordPage:
		gnome_dialog_set_sensitive(GNOME_DIALOG(mw->window),
			ZC_BUTTON_NEXT, FALSE);
		break;

	case CrackPage:

		zc_crack_stop ();
		gnome_dialog_set_sensitive(GNOME_DIALOG(mw->window),
			ZC_BUTTON_NEXT, TRUE);

		save_to_xml ();

		gtk_toggle_button_set_active
			(GTK_TOGGLE_BUTTON (mw->checkbox_startpw), TRUE);

		gtk_entry_set_text (GTK_ENTRY(mw->entry_startpw), 
				    zc_crack_get_current_password ());

		gnome_dialog_set_sensitive(GNOME_DIALOG(mw->window),
			ZC_BUTTON_NEXT, TRUE);
			
		break;
	};
	

	gtk_notebook_prev_page (GTK_NOTEBOOK (mw->notebook));
	
	if (gtk_notebook_current_page (GTK_NOTEBOOK (mw->notebook)) == 0)
		gnome_dialog_set_sensitive(GNOME_DIALOG(mw->window),
			ZC_BUTTON_PREV, FALSE);

	return TRUE;
}
/****************************************************************************/
/* Create ZipFile Page
/****************************************************************************/
static GtkWidget *
create_zipfile_page ()
{
	GtkWidget *vbox;
	GtkWidget *label;
	GtkWidget *button;
	GtkWidget *pixmap;
	GdkPixmap *icon = NULL;
	GdkBitmap *mask = NULL;
	
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vbox);
	
	label = gtk_label_new(_("Welcome to"));

	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 5);

	gtk_widget_show(label);
	
	icon = gdk_pixmap_create_from_xpm_d(mw->window->window, &mask,
		&mw->window->style->white, logo_xpm);
	
	if (icon) {
		pixmap = gtk_pixmap_new(icon, mask);
		gtk_box_pack_start(GTK_BOX(vbox), pixmap, FALSE, TRUE, 5);
		gtk_widget_show(pixmap);
	}
	
	button = gtk_button_new_with_label(_("Crack a new file"));
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 5);
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      (GtkSignalFunc) select_zipfile_cb, NULL);

	gtk_widget_show(button);
	
	button = gtk_button_new_with_label(_("Continue on the last job"));
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 5);
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
	      (GtkSignalFunc) zipcracker_continue, NULL);

      	gtk_widget_show(button);
	
	button = gtk_button_new_with_label(_("About ZipCracker"));
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 5);
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
	      (GtkSignalFunc) about, NULL);

      	gtk_widget_show(button);
							

	return vbox;
}
/****************************************************************************/
/* Creates the password page */
/****************************************************************************/
static GtkWidget *
create_password_page ()
{
	GtkWidget *vbox;
	GtkWidget *label;
	GtkWidget *table;
	
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vbox);
	
	label = gtk_label_new(_("Select the type of password:"));
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 5);
	gtk_widget_show(label);
	
	table = gtk_table_new(6, 2, FALSE);
	gtk_container_add(GTK_CONTAINER(vbox), table);
	
	mw->checkbox_az = gtk_check_button_new_with_label("a-z");
	gtk_table_attach_defaults (GTK_TABLE(table), mw->checkbox_az, 0, 1, 0, 1);
	gtk_widget_show(mw->checkbox_az);
	
	mw->checkbox_AZ = gtk_check_button_new_with_label("A-Z");
	gtk_table_attach_defaults (GTK_TABLE(table), mw->checkbox_AZ, 0, 1, 1, 2);
	gtk_widget_show(mw->checkbox_AZ);
	
	mw->checkbox_09 = gtk_check_button_new_with_label("0-9");
	gtk_table_attach_defaults (GTK_TABLE(table), mw->checkbox_09, 0, 1, 2, 3);
	gtk_widget_show(mw->checkbox_09);
	
	mw->checkbox_custom = gtk_check_button_new_with_label(_("Custom:"));
	gtk_widget_show(mw->checkbox_custom);
	
	gtk_table_attach_defaults (GTK_TABLE(table), mw->checkbox_custom, 0, 1, 3, 4);
		
	mw->entry_custom = gtk_entry_new();
	gtk_widget_set_sensitive(mw->entry_custom, FALSE);
	gtk_widget_show(mw->entry_custom);
	
	gtk_table_attach_defaults (GTK_TABLE(table), mw->entry_custom, 1, 2, 3, 4);
		
	gtk_signal_connect (GTK_OBJECT(mw->checkbox_custom), "toggled",
		    GTK_SIGNAL_FUNC (checkbox_custom_toggled_cb),
		    mw->entry_custom);
		    
	mw->checkbox_startpw = gtk_check_button_new_with_label(_("Start:"));
	gtk_widget_show(mw->checkbox_startpw);
		    
	gtk_table_attach_defaults (GTK_TABLE(table), mw->checkbox_startpw, 0, 1, 4, 5);

	mw->entry_startpw = gtk_entry_new();
	gtk_widget_set_sensitive(mw->entry_startpw, FALSE);
	gtk_widget_show(mw->entry_startpw);

	gtk_table_attach_defaults (GTK_TABLE(table), mw->entry_startpw, 1, 2, 4, 5);
		
	gtk_signal_connect (GTK_OBJECT(mw->checkbox_startpw), "toggled",
		    GTK_SIGNAL_FUNC (checkbox_startpw_toggled_cb),
		    mw->entry_startpw);
		    
	mw->checkbox_format = gtk_check_button_new_with_label(_("Format:"));
	gtk_widget_show(mw->checkbox_format);
		    
	gtk_table_attach_defaults (GTK_TABLE(table), mw->checkbox_format, 0, 1, 5, 7);

	mw->entry_format = gtk_entry_new();
	gtk_widget_set_sensitive(mw->entry_format, FALSE);
	gtk_widget_show(mw->entry_format);

	gtk_table_attach_defaults (GTK_TABLE(table), mw->entry_format, 1, 2, 5, 6);
		
	gtk_signal_connect (GTK_OBJECT(mw->checkbox_format), "toggled",
		    GTK_SIGNAL_FUNC (checkbox_format_toggled_cb),
		    mw->entry_format);
		    
	gtk_widget_show(table);
	
	return vbox;
}
/****************************************************************************/
/* Create Crack page */
/****************************************************************************/
static GtkWidget *
create_crack_page ()
{
	GtkWidget *vbox;
	GtkWidget *table;
	GtkWidget *label;
	GtkWidget *frame;
	GtkWidget *clist;
	GtkWidget *scrolled_window;
	
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vbox);
	
	mw->crack_label_filename = gtk_label_new ("");
	gtk_box_pack_start (GTK_BOX(vbox), mw->crack_label_filename, 
		FALSE, FALSE, 5);
	gtk_widget_show (mw->crack_label_filename);
	
	frame = gtk_frame_new(_("Information"));
	gtk_widget_show (frame);
	
	table = gtk_table_new(1, 2, FALSE);
	gtk_widget_show(table);
	
	label = gtk_label_new (_("Speed:"));
	gtk_label_set_justify (GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
	gtk_table_attach_defaults (GTK_TABLE(table), label, 
		0, 1, 0, 1);
	gtk_widget_show (label);
	
	mw->crack_label_speed = gtk_label_new (_("Unknown"));
	gtk_label_set_justify (GTK_LABEL(mw->crack_label_speed), 
		GTK_JUSTIFY_LEFT);
	gtk_table_attach_defaults (GTK_TABLE(table), mw->crack_label_speed, 
		1, 2, 0, 1);
	gtk_widget_show (mw->crack_label_speed);
	
	label = gtk_label_new (_("Password:"));
	gtk_label_set_justify (GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
	gtk_table_attach_defaults (GTK_TABLE(table), label, 
		0, 1, 1, 2);
	gtk_widget_show (label);

	mw->crack_label_password = gtk_label_new ("");
	gtk_label_set_justify (GTK_LABEL(mw->crack_label_password), 
		GTK_JUSTIFY_LEFT);
	gtk_table_attach_defaults (GTK_TABLE(table), mw->crack_label_password, 
		1, 2, 1, 2);
	gtk_widget_show (mw->crack_label_password);
	
	gtk_container_add (GTK_CONTAINER(frame), table);
	
	gtk_box_pack_start (GTK_BOX(vbox), frame, 
		FALSE, FALSE, 5);

	/* Create host list */
	mw->crack_clist = gtk_clist_new (2);
	gtk_clist_set_column_title (GTK_CLIST (mw->crack_clist), 0, _("Hostname"));
	gtk_clist_set_column_title (GTK_CLIST (mw->crack_clist), 1, _("Speed"));
	gtk_clist_column_titles_show (GTK_CLIST (mw->crack_clist));

	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
	gtk_container_add (GTK_CONTAINER (scrolled_window), mw->crack_clist);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
					GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);

	gtk_box_pack_start (GTK_BOX(vbox), scrolled_window, 
		TRUE, TRUE, 5);
	gtk_widget_show (mw->crack_clist);
	gtk_widget_show (scrolled_window);

	return vbox;
}
/****************************************************************************/
/* Toggles the sensitive flag on the format entry field */
/****************************************************************************/
static void
checkbox_format_toggled_cb (GtkWidget * widget, gpointer data)
{
	gtk_widget_set_sensitive(mw->entry_format, 
		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mw->checkbox_format)));
}
/****************************************************************************/
/* Toggles the sensitive flag on the custom entry field */
/****************************************************************************/
static void
checkbox_custom_toggled_cb (GtkWidget * widget, gpointer data)
{
	gtk_widget_set_sensitive(mw->entry_custom, 
		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mw->checkbox_custom)));
}
/****************************************************************************/
/* Toggles the sensitive flag on the startpw entry field */
/****************************************************************************/
static void
checkbox_startpw_toggled_cb (GtkWidget * widget, gpointer data)
{
	gtk_widget_set_sensitive(mw->entry_startpw, 
		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mw->checkbox_startpw)));
}
/****************************************************************************/
/* Free memory for filename entry */
/****************************************************************************/
static void
free_entry (gpointer data, gpointer userdata)
{
	ZipFileEntry *entry = (ZipFileEntry *)data;
	
	g_free(entry->filename);
	g_free(entry);
}
/****************************************************************************/
/* OK button in fileselection callback */
/****************************************************************************/
static void
select_zipfile_ok_cb (GtkWidget * widget, GtkWidget *fs){
	
	/* Update filename entry */
	g_string_assign(prop->zipfilename, 
		gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));

	/* Set default values */
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(mw->checkbox_az), TRUE);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(mw->checkbox_AZ), FALSE);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(mw->checkbox_09), FALSE);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(mw->checkbox_custom), FALSE);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(mw->checkbox_startpw), FALSE);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(mw->checkbox_format), FALSE);

	gtk_entry_set_text (GTK_ENTRY (mw->entry_custom), "");
	gtk_entry_set_text (GTK_ENTRY (mw->entry_startpw), "");
	gtk_entry_set_text (GTK_ENTRY (mw->entry_format), "");

	/* Select the next page */
	next_cb(NULL, NULL);
		
	/* Hide the filerequest dialog */
	gtk_widget_hide(fs);
}
/****************************************************************************/
/* Cancel button in fileselection callback */
/****************************************************************************/
static void
select_zipfile_cancel_cb (GtkWidget * widget, GtkWidget *fs){
	/* Hide the filerequest dialog */
	gtk_widget_hide(fs);
}
/****************************************************************************/
static void
select_zipfile_destroy_cb (GtkWidget * widget, GtkWidget **data){
	*data = NULL;
}
/****************************************************************************/
/* Creates the file selection dialog */
/****************************************************************************/
static void
select_zipfile_cb (GtkWidget * widget, gpointer data){
	static GtkWidget *fs = NULL;
	
	if(fs == NULL) {
		fs = gtk_file_selection_new(_("Select a zipfile"));
		
		gtk_signal_connect (GTK_OBJECT
		(GTK_FILE_SELECTION(fs)->ok_button), "clicked",
		    GTK_SIGNAL_FUNC (select_zipfile_ok_cb),
		    fs);
		gtk_signal_connect (GTK_OBJECT
		(GTK_FILE_SELECTION(fs)->cancel_button), "clicked",
		    GTK_SIGNAL_FUNC (select_zipfile_cancel_cb),
		    fs);
		gtk_signal_connect (GTK_OBJECT (fs), "destroy",
		    GTK_SIGNAL_FUNC (select_zipfile_destroy_cb),
		    &fs);
	}
	if(!GTK_WIDGET_VISIBLE(fs))
		gtk_widget_show(fs);
	else
		gtk_widget_hide(fs);
}
/****************************************************************************/

