/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  This file is part of the ZipCracker Program.

    Copyright (C) 1998-2000 Jonas Borgstrm <jonas_b@bitsmart.com>.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
/******************************************************************************/
#include <string.h>
#include <glib.h>
#include "zip.h"
/******************************************************************************/
/* Exported variables */
/******************************************************************************/
gchar *zc_fileformat_magic = NULL;
gint zc_fileformat_magic_len;
/******************************************************************************/
typedef struct _KnownFormat KnownFormat;
struct _KnownFormat {
	gchar ext[10];
	guchar magic[10];
   	gint len;
   	gint extlen;
};
/******************************************************************************/
KnownFormat knownformats[] = 
{
	{".ZIP", "PK", 2, 4},
	{".EXE", "MZ", 2, 4},
	{"", "", 0, 0}
};
/******************************************************************************/
/* Return 0 if the filename is known, else 1 */
/******************************************************************************/
static gint 
find_known_format (gconstpointer data, gconstpointer not_used)
{
	ZipFileEntry *entry = (ZipFileEntry *) data;
	gint i = 0, len = strlen(entry->filename);
	
	while(strlen(knownformats[i].ext)) {
		if(len < knownformats[i].extlen) {
		
			i++;
			continue;
		}
		if(!strncasecmp(
			&entry->filename[len - strlen(knownformats[i].ext)],
			knownformats[i].ext, knownformats[i].extlen)) {
			/* Store the values so the cracker can use them */
			zc_fileformat_magic = knownformats[i].magic;
		   	zc_fileformat_magic_len = knownformats[i].len;
			
			return 0;
		}
		i++;
	}
	return 1;	
}
/******************************************************************************/
/* Return 0 if data is "entry" is not encrypted, else 1 */
/******************************************************************************/
static gint 
find_unencrypted (gconstpointer data, gconstpointer not_used)
{
	ZipFileEntry *entry = (ZipFileEntry *) data;
	
	if(!entry->encrypted) {	

		return 0;
	} else
		return 1;
}
/******************************************************************************/
/* Return 0 if data is "entry" is compressed, else 1 */
/******************************************************************************/
static gint 
find_compressed (gconstpointer data, gconstpointer not_used)
{
	ZipFileEntry *entry = (ZipFileEntry *) data;
	
	if (entry->method == 8)
		return 0;
	else
		return 1;
}
/******************************************************************************/
/* Return 0 if data is "entry" is stored, else 1 */
/******************************************************************************/
gint 
find_stored (gconstpointer data, gconstpointer not_used)
{
	ZipFileEntry *entry = (ZipFileEntry *) data;
	
	if (entry->method == 0)
		return 0;
	else
		return 1;
}
/******************************************************************************/
/* Compares the compressed size of the files */
/******************************************************************************/
gint
static compare_compsize (gconstpointer data, gconstpointer data2)
{
	ZipFileEntry *entry = (ZipFileEntry *) data;
	ZipFileEntry *entry2 = (ZipFileEntry *) data2;
	
	return entry->comp_size - entry2->comp_size;
}
/******************************************************************************/
/* Returns the ZipFileEntry of the file best for cracking */
/******************************************************************************/
ZipFileEntry *
targetfile_get_best (GList *LIST)
{
	GList *list,*tmp_list;
	ZipFileEntry *best;
   
	zc_fileformat_magic = NULL;
	zc_fileformat_magic_len = 0;
	
	/* Make a copy of the list */
	list = g_list_copy(LIST);
	
	list = g_list_sort(list, compare_compsize);
	
	/* Remove unencrypted filenames from the list */
	while ((tmp_list = g_list_find_custom (list, NULL, find_unencrypted))) {

		list = g_list_remove_link(list, tmp_list);
	}
		
	if(list == NULL)
		return NULL;
	
	/* Remove all compressed files if there are uncomressed files */
	if( g_list_find_custom (list, NULL, find_stored)) {
	
		while ((tmp_list = g_list_find_custom (list, 
			NULL, find_compressed))) {
	
			list = g_list_remove_link(list, tmp_list);
		}
	
	}
	
	/* It the format is know then use that file, else the smallest file */
	if( g_list_find_custom (list, NULL, find_known_format))
		best = (ZipFileEntry *)g_list_find_custom (list, NULL,
			find_known_format)->data;			
	else	
		best = (ZipFileEntry *)g_list_first(list)->data;
	
	/* g_print("Best Choice = '%s'\n", best->filename);*/
	
	return best;
}
/******************************************************************************/
