/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  This file is part of the ZipCracker Program.

    Copyright (C) 1998-2000 Jonas Borgstrm <jonas_b@bitsmart.com>.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
#include <glib.h>
#include <stdio.h>
#include <string.h>
#include "zip.h"


/* Macros for getting two-byte and four-byte header values */
#define SH(p) ((unsigned short)(unsigned char)((p)[0]) | ((unsigned short)(unsigned char)((p)[1]) << 8))
#define LG(p) ((unsigned long)(SH(p)) | ((unsigned long)(SH((p)+2)) << 16))

static const unsigned char zip_sig[4] = {0x50, 0x4b, 0x03, 0x04}; /* Sign of zip header */
/*****************************************************************************/
/*    Parse zipfile to get CRC value */
/*****************************************************************************/
GList *
parse_zipfile (const gchar *filename)
{ 
	unsigned char zipheader[28]; /* Local zip header */
	unsigned char buffer[1024];
	FILE *zipfd;
       	int i, end;
  	GList *list = NULL;
   	ZipFileEntry *entry;
	
	/* Uses "rb" for compability with msdos */
	if((zipfd = fopen(filename, "rb")) == NULL) {
		
		return NULL;
	}
	
	i = 0;
	end=0;	
	
	while(!end) {
		if(fread(buffer, 1, 1, zipfd)) {
		   
			if(buffer[0] == zip_sig[i])
				i = i + 1;
			else
				i = 0;
			
			if (i == 4) {

				if(fread(zipheader + 2, 1, 26, zipfd) != 26)
					return NULL;

				if(fread(buffer, 1, SH(zipheader + 24), zipfd)) {
				
					buffer[SH(zipheader + 24)] = 0;
					entry = g_malloc0(sizeof(ZipFileEntry));
					entry->filename = g_strdup(buffer);
				   
					i=0;
					/* Read local header */
					fread(buffer, 1, zipheader[26], zipfd);
					entry->orig_size  = LG(zipheader + 20);
					
					entry->comp_size = LG(zipheader + 16);
					entry->method = zipheader[6];
					entry->encrypted = zipheader[4] & 1;
					
					/* Is file encrypted? */
					if(zipheader[4] & 1) {
						/* Read random header */
					        fread(entry->random_header, 1, 12, zipfd);
						/* Get crc and original & compressed size */
						entry->crc32 = LG(zipheader + 12);
						entry->comp_size -= 12;
						
						/* Have extended header? */
						if(zipheader[4] & 8)
							entry->header_crc = zipheader[9];
						else
							entry->header_crc = zipheader[12 + 3];
					}

					entry->file_offset = ftell(zipfd);
					
					fseek(zipfd, entry->comp_size, SEEK_CUR);
					
					list = g_list_append(list, entry);
				}
				else
					return NULL;
			}
		}
		else
			end = 2;
	}
	fclose(zipfd);
	
	return list;
}
