/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  This file is part of the ZipCracker Program.

    Copyright (C) 1998-2000 Jonas Borgstrm <jonas_b@bitsmart.com>.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#include <glib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <pvm3.h>

#include "zipcracker.h"
#include "crack.h"
/*****************************************************************************/
gchar *zc_fileformat_magic = NULL;
gint zc_fileformat_magic_len;
/*****************************************************************************/
static guint8 header_crc;
static guint8 header[12];
static guint8 method;
static gulong comp_size, orig_size;
static gulong CRC32;
static FILE *tmpfd;
static int ptid;
/*****************************************************************************/
extern gchar zc_format[PW_LEN];
extern gchar zc_string[PW_LEN];
/*****************************************************************************/
/*   Initializes PVM, returns TRUE on sucess */
/*****************************************************************************/
static gboolean
init_pvm ()
{
	struct timeval wait_time;
	guint msgtag, version;
	gchar hostname[64];
	int cc;
	gulong filesize, packetsize;
	guint8 buffer[4096];
	
	/* 1/2 minute timeout */
	wait_time.tv_sec = 300;
	wait_time.tv_usec = 0;
	
	pvm_initsend (PvmDataDefault);
	
	ptid = pvm_parent ();
	
	/* Have PVM notify us if the host zipcracker process is killed
	 * or the local PVM daemon dies */
	pvm_notify (PvmTaskExit, PVM_KILL_SLAVE, 1, &ptid);
	pvm_setopt (PvmAutoErr, 2);
	
	/* Init structure */
	version = 001;
	gethostname (hostname, 63);

	/* Sending Info */
	pvm_pkuint (&version, 1, 1);
	pvm_pkstr (hostname);
	pvm_send (ptid, PVM_INIT);

	/* Reading ack */
	if((cc = pvm_trecv (ptid, -1, &wait_time)) == 0) {

		g_print ("Slave timed out waiting for master\n");
		pvm_exit ();
		exit (RETURN_SLAVE_TIMED_OUT);
	}	
	pvm_bufinfo (cc, NULL, &msgtag, &ptid);
	if (msgtag != PVM_INIT) {

		g_print ("Unknown msgtag %u from master\n", msgtag);
		pvm_exit ();
		exit (RETURN_UNKNOWN_MSGTAG);
	}
	pvm_upkstr (zc_string);
	pvm_upkstr (zc_format);
	pvm_upkint (&zc_fileformat_magic_len, 1, 1);
	zc_fileformat_magic = g_malloc (zc_fileformat_magic_len);
	pvm_upkbyte (zc_fileformat_magic, zc_fileformat_magic_len, 1);

	pvm_upkbyte (&header_crc, 1, 1);
	pvm_upkulong (&CRC32, 1, 1);
	pvm_upkbyte (header, 12, 1);
	pvm_upkbyte (&method, 1, 1);
	pvm_upkulong (&comp_size, 1, 1);
	pvm_upkulong (&orig_size, 1, 1);

	tmpfd = tmpfile ();

	/* Read the encryped data from the server */
	filesize = comp_size;	

	while (filesize) {
		fflush (tmpfd);
		/* Reading packed */
		if((cc = pvm_trecv (ptid, -1, &wait_time)) == 0) {
			
			g_print ("Slave timed out waiting for master\n");
			pvm_exit ();
			exit (RETURN_SLAVE_TIMED_OUT);
		}	
		pvm_bufinfo (cc, NULL, &msgtag, &ptid);
		if (msgtag != PVM_FILE_PACKET) {
			
			g_print ("Unexpected msgtag %u from master\n", msgtag);
			pvm_exit ();
			exit (RETURN_UNEXPECTED_MSGTAG);
		}
		pvm_upkulong (&packetsize, 1, 1);
		pvm_upkbyte (buffer, packetsize, 1);
		/* Write the info to the temp file */
		fwrite (buffer, packetsize, 1, tmpfd);

		filesize -= packetsize;
	}

	return TRUE;
}
/*****************************************************************************/
/*   This function gets called when we found the password */
/*****************************************************************************/
static void
found_cb (gchar *password)
{
	/* Send the password to the server */
	pvm_initsend(PvmDataDefault);
	pvm_pkstr(password);
	pvm_send(ptid, PVM_FOUND);
}
/*****************************************************************************/
/*   zipcracker_slave entry point */
/*****************************************************************************/
int
main (int argc, char **argv)
{
	gulong rate = 0;
	struct timeval wait_time;
	gint cc, count;
	gchar start[PW_LEN];
	gchar end[PW_LEN];
	guint msgtag;
	gulong block_time;
	GTimeVal ts, ts2;

	/* Check to see if pvm invoked us */
	if (argc != 2 || strcmp ("--zipcracker_slave", argv[1])) {

		g_print ("zipcracker_slave version %s:\nThis program should only be invoked by zipcracker.\n", VERSION);
		return RETURN_OK;
	}

	/* 1/2 minute timeout */
	wait_time.tv_sec = 30;
	wait_time.tv_usec = 0;
	
	init_pvm ();

	while(1) {
		/* Requesting new block */
		pvm_initsend(PvmDataDefault);
		pvm_pkulong(&rate, 1, 1);
		pvm_pkulong(&block_time, 1, 1);
		pvm_send(ptid, PVM_REQUEST);

		/* Reading ack */
		if((cc = pvm_trecv(ptid, -1, &wait_time)) == 0) {
			g_print ("Slave timed out waiting for master\n");
			pvm_exit ();
			exit (RETURN_SLAVE_TIMED_OUT);
		}	
		pvm_bufinfo(cc, NULL, &msgtag, &ptid);

		switch(msgtag) {

		case PVM_KILL_SLAVE:
			/* Server want us to quit? */
			pvm_exit ();
			exit (RETURN_OK);
			break;

		case PVM_REQUEST:
			/* Get info from the server */
			pvm_upkstr (start);
			pvm_upkstr (end);
			/* Save start time */
			/*			time (&timestamp);*/
			g_get_current_time (&ts);

			count = crack (start, end, zc_format, zc_string, 
				       header_crc, header, method, 
				       0, orig_size, comp_size, CRC32, 
				       tmpfd, found_cb);

			g_get_current_time (&ts2);

			/* Calculate the rate */
			block_time = (ts2.tv_sec - ts.tv_sec) * 1000 + (ts2.tv_usec - ts.tv_usec) / 1000;

			/* Avoid division by zero */
			if (block_time)
				rate = count * 1000 / block_time;
			else 
				rate = 0;

			break;
		};
	};

	fclose (tmpfd);

	if (zc_fileformat_magic)
		g_free (zc_fileformat_magic);

	return RETURN_OK;
}
