/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  This file is part of the ZipCracker Program.

    Copyright (C) 1998-2000 Jonas Borgstrm <jonas_b@bitsmart.com>.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
#include <glib.h>
#include <stdio.h>
#include <string.h>
#include <zlib.h>

#include "zipcracker.h"
#include "crack.h"
#include "crc_table.h"
#include "zip.h"
/*****************************************************************************/
extern gchar *zc_fileformat_magic;
extern gint zc_fileformat_magic_len;
/*****************************************************************************/
/* Global variables */
/*****************************************************************************/
gchar zc_format[PW_LEN];
gchar zc_string[PW_LEN];
gchar zc_passwd[PW_LEN];
/*****************************************************************************/
static char *format_passwd (const char *guess, char *realpw, const char *format);
static void update_keys (unsigned long *key, char val);
static unsigned char decrypt_byte (const unsigned long *key);
static int decript_head (unsigned char *buffer, unsigned long *key, guint8 header_crc);
static unsigned long Crc32 (unsigned long oldcrc, gchar newchar);
static gboolean test_password (unsigned long *key, guint8 method, gulong file_offset, gulong orig_size, gulong comp_size, gulong CRC32, FILE *fd);
/*****************************************************************************/
/****************************************************************************/
/*    Main cracking procedure, returns the number of passwords tested */
/****************************************************************************/
gint
crack (const char *start, 
       const char *end, 
       const char *format,
       const char *string,
       guchar header_crc,
       guint8 *header,
       guint8 method,
       gulong file_offset,
       gulong orig_size,
       gulong comp_size,
       gulong CRC32,
       FILE *fd,
       void (*found_cb) (gchar *))
{
	gchar passwd[PW_LEN];
	gchar realpw[PW_LEN]; /* Passwd after running format_passwd */
	gint i, len, count = 0;
	unsigned long key[3];
	unsigned char hackbuff[19];

	strcpy (passwd, start);
	
   	len = strlen(passwd);
	
	for(;;) {

		count++;

		zc_format_passwd (passwd, realpw, format);
		
		memcpy(hackbuff, header, 12);
		key[0] = 305419896L;
		key[1] = 591751049L;
		key[2] = 878082192L;
		
		for(i=0; i<strlen(realpw);i++)
			update_keys(key,realpw[i]);

		/* Is realpw a possible password */
		if(decript_head (hackbuff, key, header_crc)) {
			
			if(test_password (key, method, file_offset, orig_size, comp_size, CRC32, fd)) {
				(*found_cb)(passwd);
				return count;
			}		
		}
		/* Done enough? */
		if(!strcmp (passwd, end))
			return count;

		i = len - 1;
		passwd[i] = *(strchr(string, passwd[i]) + 1);

   		while(passwd[i] == 0) {
		   	if(i == 0) {
				
			   	memmove(passwd + 1, passwd, len);
			   	passwd[0] = *string;
			   	passwd[1] = *string;
			   	len++;
			   	passwd[len] = 0;
				break;	
				
			} else {
				passwd[i] = *string;
				passwd[i - 1] = *(strchr(string, passwd[i - 1]) + 1);
				i--;
				if(i == -1)
					break;
			}
		};
	}
	return count;
}
/*****************************************************************************/
/*    Tries to decrypt header and checks if CRC match, if so return 1 */
/*****************************************************************************/
static int 
decript_head(unsigned char *buffer, unsigned long *key, guint8 header_crc)
{
	int i;
	char c;
	
	for(i = 0; i<12;i++) {
		c = buffer[i] ^ decrypt_byte(key);
		update_keys(key,c);
		buffer[i] = c;
	}
	
	return buffer[11] == header_crc;
}
/*****************************************************************************/
/*    update_keys() */
/*****************************************************************************/
static void
update_keys(unsigned long *key, char val)
{
	
	key[0] = Crc32(key[0], val);
	key[1] = (key[1] + (key[0] & 0xff)) * 134775813L + 1;
	key[2] = Crc32(key[2], (key[1] >> 24));
}
/*****************************************************************************/
/*    decrypt_byte */
/*****************************************************************************/
static unsigned char 
decrypt_byte(const unsigned long *key) {
	unsigned short temp;
	
	temp = key[2] | 2;
	return((temp *( temp^1)) >> 8);
}
/*****************************************************************************/
/*    CRC rutine */
/*****************************************************************************/
static unsigned long 
Crc32 (unsigned long oldcrc, gchar newchar)
{
	register unsigned long Crc = oldcrc;
	
        Crc = ((Crc >> 8) & 0x00ffffffL) ^ 
		crc_table[(Crc ^ newchar) & 0xffL];
	
	return Crc;            /* return a 1's complement */
}
/*****************************************************************************/
/*    Test if the password is correct, by unpacking and testing CRC */
/*****************************************************************************/
static gboolean
test_password (unsigned long *key, 
	       guint8 method,
	       gulong file_offset,
	       gulong orig_size,
	       gulong comp_size,
	       gulong CRC32,
	       FILE *fd)
{
	gchar inbuffer[REAL_BUF_SIZE];
	gchar outbuffer[REAL_BUF_SIZE];
	int err = Z_OK, i, n;
	int in_buf_size, out_buf_size;
	struct z_stream_s zstrm;
	char c;
	unsigned long newcrc;
	long bl;
   	gboolean check_fileformat_magic;


	check_fileformat_magic = zc_fileformat_magic_len > 0;
   
	out_buf_size = (zc_fileformat_magic_len > 0) ? (zc_fileformat_magic_len) : REAL_BUF_SIZE;
   
	/* Initilize CRC */
	newcrc = crc32 (0L, Z_NULL, 0);
		
	/* Check compresion method */
   
	if(method == DEFLATED) {
		fseek (fd, file_offset, SEEK_SET);
		
		zstrm.avail_in = fread (inbuffer, 1, (comp_size < REAL_BUF_SIZE) ? comp_size : REAL_BUF_SIZE, fd);
		
		for(i = 0; i<zstrm.avail_in;i++) {

			c = inbuffer[i] ^ decrypt_byte(key);
			update_keys (key,c);
			inbuffer[i] = c;
		}
		
		zstrm.next_out = outbuffer;
		zstrm.avail_out = out_buf_size;
		zstrm.next_in = inbuffer;
		zstrm.zalloc = (alloc_func)Z_NULL;
		zstrm.zfree = (free_func)Z_NULL;
		zstrm.opaque = (free_func)Z_NULL;
		
		/* Initilize zlib */
		if((err = inflateInit2 (&zstrm, -15)) != Z_OK ) {
			
			g_error("inflateInit() failed (%d)\n", err);
		}
		
		for(;;) {			
			err = inflate (&zstrm, Z_NO_FLUSH);
			/* Did it go wrong? */
			if (err < 0) {
				
				inflateEnd (&zstrm);
				return FALSE;
			}
			if(check_fileformat_magic && zstrm.avail_out == 0) {

				check_fileformat_magic = FALSE;

				if(memcmp (outbuffer, zc_fileformat_magic, zc_fileformat_magic_len) != 0) {
					/* No can't be correct password */
					inflateEnd (&zstrm);
					return FALSE;
				}
				/* Yes might be correct password */
			   	in_buf_size = REAL_BUF_SIZE;
			}
			if(zstrm.avail_in == 0) {
				zstrm.avail_in = fread (inbuffer, 1, ((comp_size - zstrm.total_in)< REAL_BUF_SIZE) ? 
						       (comp_size - zstrm.total_in) : REAL_BUF_SIZE, fd);

				for(i = 0; i < zstrm.avail_in;i++) {

					c = inbuffer[i] ^ decrypt_byte(key);
					update_keys (key,c);
					inbuffer[i] = c;
				}
				zstrm.next_in = inbuffer;
			}
			/* Run CRC on the output */
			newcrc = crc32 (newcrc, outbuffer, out_buf_size - zstrm.avail_out);
			zstrm.next_out = outbuffer;
			zstrm.avail_out = out_buf_size = REAL_BUF_SIZE;
	
			/* Are we finished? */
			if (err == Z_STREAM_END)
				break;			
		}
		inflateEnd (&zstrm);

	} else if (method == STORED) {

		in_buf_size = (zc_fileformat_magic_len > 0) ? zc_fileformat_magic_len : REAL_BUF_SIZE;
		fseek (fd, file_offset, SEEK_SET);
		bl = orig_size;
		
		while ((n = fread (inbuffer, 1, (bl > in_buf_size) ? in_buf_size : bl, fd))) {
			bl -= n;
			/* Decrypt */
			for (i = 0; i<n;i++) {
				c = inbuffer[i] ^ decrypt_byte(key);
				update_keys (key,c);
				inbuffer[i] = c;
			}
		   	if (check_fileformat_magic) {
				check_fileformat_magic = FALSE;
				
				if(memcmp (inbuffer, zc_fileformat_magic, zc_fileformat_magic_len) != 0)
					/* No can't be correct password */
			     		return FALSE;
								
			   	in_buf_size = REAL_BUF_SIZE;
			   	out_buf_size = REAL_BUF_SIZE;
			}
			newcrc = crc32 (newcrc, inbuffer, n);
		};
	}
					
	return(CRC32 == newcrc);
}
/****************************************************************************/
/*    Formats the real password */
/****************************************************************************/
char *
zc_format_passwd (const char *guess, char *realpw, const char *format) 
{
	int i, j = 0, k = 0;

	for(i=0;i<strlen(format);i++) {
		switch(format[i]) {
		case '?':
			if(guess[k])
				realpw[j++] = guess[k++];
			break;
		case '*':
			while(guess[k] != 0) 
				realpw[j++] = guess[k++];
			break;
		default:
			realpw[j++] = format[i];
		};
		/* Don't overrun the memory limit */
		if (j >= (PW_LEN -1))
			break;
	}	
	realpw[j] = 0;
	
	return realpw;
}
/*****************************************************************************/
/* Returns a pointer to a static buffer containing the current password */
/*****************************************************************************/
gchar *
zc_crack_get_current_password_formatted ()
{
	static gchar buffer[PW_LEN];

	zc_format_passwd (zc_passwd, buffer, zc_format);
	
	return buffer;
}
/*****************************************************************************/
/* Returns a pointer to a static buffer containing the current password */
/*****************************************************************************/
gchar *
zc_crack_get_current_password ()
{
	return zc_passwd;
}
/****************************************************************************/
