/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  This file is part of the ZipCracker Program.

    Copyright (C) 1998-2000 Jonas Borgstrm <jonas_b@bitsmart.com>.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
#include <gnome.h>
#include <stdio.h>
#include <string.h>
#include <zlib.h>
#include <pthread.h>

#ifdef USE_PVM
#include <pvm3.h>
#endif

#include "zipcracker.h"
#include "mainwin.h"
#include "crack_server.h"
/*****************************************************************************/
static guchar header_crc;
static guchar *header;
static guint8 method;
static gulong file_offset;
static gulong orig_size, comp_size;
static gulong CRC32;

static pthread_t crackthread;
static int crack_timer_id = -1;
static GHashTable *slaves;
static gulong total_rate;
static gboolean flag_found_correct_password, flag_stop_cracking;
GList *zc_todo = NULL;
/*****************************************************************************/
extern gchar *zc_fileformat_magic;
extern gint zc_fileformat_magic_len;
extern Properties *prop;
extern MainWindow *mw;
extern gchar zc_format[PW_LEN];
extern gchar zc_string[PW_LEN];
extern gchar zc_passwd[PW_LEN];
/****************************************************************************/
/*****************************************************************************/
static gint crackpage_timeout (gpointer data);
/*****************************************************************************/
/*   Updates passwd i.e: a -> b,  bb -> cb */
/*****************************************************************************/
static void 
passwd_inc (char *guess, const char *string)
{
	int i, len = strlen (guess);
	
	i = len - 1;
	guess[i] = *(strchr (string, guess[i]) + 1);
	
	while(guess[i] == 0) {
		if(i == 0) {
			
			memmove(guess + 1, guess, len);
			guess[0] = string[0];
			guess[1] = string[0];
			len++;
			guess[len] = 0;
			break;	
			
		} else {
			
			guess[i] = string[0];
			guess[i - 1] = *(strchr(string, guess[i - 1]) + 1);
			i--;
			if(i == -1)
				break;
		}
	}
}
/*****************************************************************************/
/*   Updates passwd i.e a -> aaaaaa */
/*****************************************************************************/
static void 
passwd_inc_much(char *guess, const char *string, int block_size)
{
	int i, len;

	/* If password is to short, make it longer */
	if(strlen (guess) < block_size) {
		len = strlen(guess);
		for(i=len;i < block_size;i++) {
			guess[i] = string[0];
		}
		guess[block_size] = 0;
		return;
	}
	
	len = strlen(guess);

	i = len - block_size;

	guess[i] = *(strchr (string, guess[i]) + 1);
	while(guess[i] == 0) {
		if(i == 0) {
			
			memmove(guess + 1, guess, len);
			guess[0] = string[0];
			guess[1] = string[0];
			len++;
			guess[len] = 0;
			break;	
			
		} else {
			
			guess[i] = string[0];
			guess[i - 1] = *(strchr(string, guess[i - 1]) + 1);
		}
		i--;
		if(i == -1)
			break;
	}
}
/*****************************************************************************/
/*    Tells the slave to die, and frees all the memory */
/*****************************************************************************/
static gboolean
stop_slave_and_free (gpointer key, gpointer value, gpointer user_data)
{
	ZCSlave *entry = (ZCSlave *)value;
#ifdef USE_PVM
	/* Sending "Kill packet" to the slave */
	pvm_initsend (PvmDataDefault);
	pvm_send (entry->tid, PVM_KILL_SLAVE);	
#endif
	g_free (entry);

	return TRUE;
}
/*****************************************************************************/
/*    Add blocks to todo list */
/*****************************************************************************/
static void
add_to_todo (gpointer key, gpointer value, gpointer user_data)
{
	ZCSlave *entry = (ZCSlave *)value;
	ZCTodo *t = g_new (ZCTodo, 1);
	
	strcpy (t->start, entry->start);
	strcpy (t->end, entry->end);

	zc_todo = g_list_append (zc_todo, t);
}
/*****************************************************************************/
/*    PVM:Waits for the clients and sends ack. */
/*****************************************************************************/
#ifdef USE_PVM
static void
crack_main_loop_pvm (FILE *zipfd)
{
	struct timeval wait_time;
	guint msgtag, version;
	gulong filesize, packetsize;
	gchar hostname[64];
	int tid, cc, i;
	guint8 buffer[4096];
	ZCSlave *entry;

	/* 10 minute timeout */
	wait_time.tv_sec = 60;
	wait_time.tv_usec = 0;


	slaves = g_hash_table_new (NULL, NULL);
	
	while (flag_stop_cracking == FALSE) {

		/* Wait for packages */
		if((cc = pvm_trecv (-1, -1, &wait_time)) == 0) {
			printf(_("Server timed out waiting for slaves\n"));
			pvm_exit ();
			exit (RETURN_MASTER_TIMED_OUT);
		}
		pvm_bufinfo(cc, NULL, &msgtag, &tid);

		switch (msgtag) {
			/* The first packet sent by the slave */
		case PVM_INIT:
			/* Get the info from the slave */
			pvm_upkuint (&version, 1, 1);
			pvm_upkstr (hostname);

			entry = g_new (ZCSlave, 1);
			/* Fill the slave structure */
			strcpy (entry->hostname, hostname);
			entry->block_size = 4;
			entry->nblocks = 1;
			entry->tid = tid;
			entry->block_time = 0;
			/* Add the slave to the hash */
			g_hash_table_insert (slaves, GINT_TO_POINTER (tid), entry);

			/* Send variables */
			pvm_initsend (PvmDataDefault);
			pvm_pkstr (zc_string);
			pvm_pkstr (zc_format);
			pvm_pkint (&zc_fileformat_magic_len, 1, 1);
			pvm_pkbyte (zc_fileformat_magic, zc_fileformat_magic_len, 1);
			pvm_pkbyte (&header_crc, 1, 1);
			pvm_pkulong (&CRC32, 1, 1);
			pvm_pkbyte (header, 12, 1);
			pvm_pkbyte (&method, 1, 1);
			pvm_pkulong (&comp_size, 1, 1);
			pvm_pkulong (&orig_size, 1, 1);
			pvm_send (tid, PVM_INIT);

			/* Send the encrypted data to the slave */
			fseek(zipfd, file_offset, SEEK_SET);

			filesize = comp_size;

			while (filesize) {

				packetsize = (filesize < 4096) ? filesize : 4096;
				fread (buffer, packetsize, 1, zipfd);
				filesize -= packetsize;

				pvm_initsend (PvmDataDefault);
				pvm_pkulong (&packetsize, 1, 1);
				pvm_pkbyte (buffer, packetsize, 1);

				pvm_send (tid, PVM_FILE_PACKET);
			}

			break;
		case PVM_REQUEST:
			/* The slave asks for a new block */
			/* Find the slaves entry in the hash */
			entry = g_hash_table_lookup (slaves, GINT_TO_POINTER (tid));
			/* Get the rate from the slave */
			pvm_upkulong (&entry->rate, 1, 1);
			pvm_upkulong (&entry->block_time, 1, 1);

			/* Update the block size */
			if (entry->block_time) {

				if (entry->block_time < MIN_CRACK_TIME) {

					entry->nblocks++;
					if (entry->nblocks > strlen (zc_string)) {
						entry->block_size++;
						entry->nblocks = 1;
					}
				} else if (entry->block_time > MAX_CRACK_TIME) {
					entry->nblocks--;
					if (entry->nblocks == 0) {
						if (entry->block_size > 1) {
							entry->block_size--;
							entry->nblocks = strlen (zc_string);
						} else
							entry->nblocks = 1;
					}
				}
			}

			pvm_initsend (PvmDataDefault);

			/* Use an old block or create a new */
			if (zc_todo) {
				ZCTodo *t = (ZCTodo *)zc_todo->data;

				pvm_pkstr (t->start);
				pvm_pkstr (t->end);
				strcpy (entry->start, t->start);
				strcpy (entry->end, t->end);

				/* Free todo struct */
				zc_todo = g_list_remove (zc_todo, t);
				g_free (t);

				
			} else {
				pvm_pkstr (zc_passwd);
				strcpy (entry->start, zc_passwd);
				
				for (i=0;i < entry->nblocks; i++)
					passwd_inc_much (zc_passwd, zc_string, entry->block_size);
				
				pvm_pkstr (zc_passwd);
				strcpy (entry->end, zc_passwd);	
				passwd_inc (zc_passwd, zc_string);
			}
			pvm_send (tid, PVM_REQUEST);

			break;
		case PVM_FOUND:
			/* The slave has found the password */
			/* Find the slaves entry in the hash */
			entry = g_hash_table_lookup (slaves, GINT_TO_POINTER (tid));

			pvm_upkstr (zc_passwd);
			flag_found_correct_password = TRUE;
			flag_stop_cracking = TRUE;

			break;
		};

	}
	/* Save the unfinished blocks */
	g_hash_table_foreach (slaves, add_to_todo, NULL);
	/* free the slaves :) */
	g_hash_table_foreach_remove (slaves, stop_slave_and_free, NULL);
}
/*****************************************************************************/
/* Initializes PVM and spawns the tasks, returns TRUE on success */
/*****************************************************************************/
static gboolean
crack_init_pvm ()
{
	int nhosts, cc;
	int ctid[MAX_SLAVES];
	char *args[2];

	/* Disable error messages */
	pvm_setopt(PvmAutoErr, 0);

	if(pvm_mytid() < 0)
		return FALSE;
	
	pvm_config (&nhosts, NULL, NULL);

	args[0] = "--zipcracker_slave";
	args[1] = NULL;

	cc = pvm_spawn ("zipcracker_slave", args, 0, "", nhosts, ctid);
	
	if (cc > 1)
		return TRUE;
	else 
		return FALSE;
}
#endif
/*****************************************************************************/
/*   This function gets called when we found the password */
/*****************************************************************************/
static void
found_cb (gchar *password)
{
	strcpy (zc_passwd, password);

	flag_found_correct_password = TRUE;
	flag_stop_cracking = TRUE;
}
/*****************************************************************************/
/*    PVM:Waits for the clients and sends ack. */
/*****************************************************************************/
static void
crack_main_loop (FILE *zipfd)
{
	ZCSlave *entry;
	gchar start[PW_LEN], end[PW_LEN];
	gint i, count;
	GTimeVal ts, ts2;

	/* Add a dummy entry */
	slaves = g_hash_table_new (NULL, NULL);

	entry = g_new (ZCSlave, 1);
	/* Fill the slave structure with default values */
	gethostname (entry->hostname, 63);
	entry->block_size = 4;
	entry->nblocks = 1;
	entry->tid = 1;
	entry->block_time = 0;
	/* Add the slave to the hash */
	g_hash_table_insert (slaves, GINT_TO_POINTER (1), entry);
	
	while (flag_stop_cracking == FALSE) {
		
		/* Update the block size */
		if (entry->block_time) {
			
			if (entry->block_time < MIN_CRACK_TIME) {
				
				entry->nblocks++;
				if (entry->nblocks > strlen (zc_string)) {
					entry->block_size++;
					entry->nblocks = 1;
				}
			} else if (entry->block_time > MAX_CRACK_TIME) {
				entry->nblocks--;
				if (entry->nblocks == 0) {
					if (entry->block_size > 1) {
						entry->block_size--;
						entry->nblocks = strlen (zc_string);
					} else
						entry->nblocks = 1;
				}
			}
		}
		
		/* Use an old block or create a new */
		if (zc_todo) {
			ZCTodo *t = (ZCTodo *)zc_todo->data;
			
			strcpy (start, t->start);
			strcpy (end, t->end);
			
				/* Free todo struct */
			zc_todo = g_list_remove (zc_todo, t);
			g_free (t);
			
			
		} else {
			strcpy (start, zc_passwd);
			
			for (i=0;i < entry->nblocks; i++)
				passwd_inc_much (zc_passwd, zc_string, entry->block_size);
			
			strcpy (end, zc_passwd);
			passwd_inc (zc_passwd, zc_string);
			
		}
		g_get_current_time (&ts);

		count = crack (start, end, zc_format, zc_string, 
			       header_crc, header, method, 
			       file_offset, orig_size, comp_size, CRC32, 
			       zipfd, found_cb);
		
		g_get_current_time (&ts2);

		/* Calculate the rate */
		entry->block_time = (ts2.tv_sec - ts.tv_sec) * 1000 + (ts2.tv_usec - ts.tv_usec) / 1000;

		/* Avoid division by zero */
		if (entry->block_time)
			entry->rate = count * 1000 / entry->block_time;
		else 
			entry->rate = 0;
		
	}
	/* free the slaves :) */
	g_hash_table_foreach_remove (slaves, stop_slave_and_free, NULL);
}
/*****************************************************************************/
/* Find todo entrie that are invalid (contains invalid characters) */
/*****************************************************************************/
static gint
todo_find_invalid (gconstpointer data, gconstpointer user_data)
{
	gint i;
	ZCTodo *t = (ZCTodo *)data;
	
	i = 0;

	while (t->start[i]) {
		if (strchr (zc_string, t->start[i]) == NULL)
			return 0;

		i++;
	}
	i = 0;

	while (t->end[i]) {
		if (strchr (zc_string, t->end[i]) == NULL)
			return 0;

		i++;
	}
	return 1;
}
/*****************************************************************************/
/* Entry point of the crack thread */
/*****************************************************************************/
static void *
crack_thread (void *arg)
{
	ZipFileEntry *target = (ZipFileEntry *)arg;
	FILE *zipfd = NULL;
	gchar start[PW_LEN], end[PW_LEN];
	GList *tmplist;
	
   	header_crc 	= target->header_crc;
	header 		= target->random_header;
	method 		= target->method;
	file_offset	= target->file_offset;  	
	comp_size	= target->comp_size;  	
	orig_size	= target->orig_size;  	
	CRC32		= target->crc32;

	strcpy (zc_string, zc_get_password_string ());
	strcpy (zc_format, zc_get_password_format ());
	strcpy (zc_passwd, zc_get_start_password ());

	flag_stop_cracking = FALSE;

 	if((zipfd = fopen (prop->zipfilename->str, "rb")) == NULL) {
	   
		g_print (_("Can't open file '%s'\n"), prop->zipfilename->str);
	   
	   	exit (0);
	}

	/* Remove all invalid blocks from the todo list */
	if (zc_todo) {
		while ((tmplist = g_list_find_custom (zc_todo, NULL, todo_find_invalid))) {
			
			ZCTodo *t = (ZCTodo *)tmplist->data;
			
			zc_todo = g_list_remove (zc_todo, tmplist->data);
		}
	}

#ifdef USE_PVM
	if (crack_init_pvm ()) {

		crack_main_loop_pvm (zipfd);
		pvm_exit();
	} else {
#endif
		crack_main_loop (zipfd);
#ifdef USE_PVM
	}
#endif
   	fclose(zipfd);

	return NULL;
}
/*****************************************************************************/
/* Stops cracking */
/*****************************************************************************/
void
zc_crack_stop ()
{
	/* Stop the timer */
	if (crack_timer_id != -1) {

		gtk_timeout_remove (crack_timer_id);
		crack_timer_id = -1;
	}
	/* Tell the thread to die */
	flag_stop_cracking = TRUE;
	/* Wait for it to die */
	pthread_join(crackthread, NULL);	
}
/*****************************************************************************/
/* Entry point of the crack thread */
/*****************************************************************************/
gboolean
zc_crack_start (ZipFileEntry *target)
{
	flag_found_correct_password = FALSE;

	pthread_create(&crackthread, NULL, crack_thread, (void *)target);
	
	crack_timer_id = gtk_timeout_add(1000, crackpage_timeout, NULL);

}
/*****************************************************************************/
/*    Tells the slave to die, and frees all the memory */
/*****************************************************************************/
static void
add_slave_to_clist (gpointer key, gpointer value, gpointer user_data)
{
	gchar *text[3];
	gchar rate_text[40];
	GtkCList *clist = GTK_CLIST (user_data);
	gint row;

	ZCSlave *entry = (ZCSlave *)value;

	text[0] = entry->hostname;
	text[1] = rate_text;
	text[2] = NULL;

	sprintf (rate_text, "%lu", entry->rate);
	total_rate += entry->rate;
	
	row = gtk_clist_append (clist, text);
}
/****************************************************************************/
/* Updates the status/crack page */
/****************************************************************************/
static gint
crackpage_timeout (gpointer data)
{
	gchar message[PW_LEN * 2];

	/* Updates Password label */	

	gtk_label_set_text ( 
		GTK_LABEL (mw->crack_label_password), 
		zc_crack_get_current_password_formatted ());

	/* Update Speed label */
	sprintf (message, "%d", total_rate);
		
	gtk_label_set_text ( 
		GTK_LABEL (mw->crack_label_speed), message);		

	if(flag_found_correct_password) {

		/* We don't need to update the screen anymore */
		gtk_timeout_remove (crack_timer_id);
		crack_timer_id = -1;
		
		/* Create the dialog box */
		sprintf (message, _("Found correct password\nPassword = '%s'"),
			 zc_crack_get_current_password_formatted ());
			
		gtk_widget_show (gnome_message_box_new (message, 
							GNOME_MESSAGE_BOX_INFO, 
							GNOME_STOCK_BUTTON_OK, 
							NULL));
		return FALSE;
	}

	/* Clear the clist and add the entries from the hash again */
	gtk_clist_clear (GTK_CLIST (mw->crack_clist));
	total_rate = 0;
	g_hash_table_foreach (slaves, add_slave_to_clist, mw->crack_clist);
	gtk_clist_columns_autosize (GTK_CLIST (mw->crack_clist));
	
	return TRUE;
}
/****************************************************************************/
